@@ -26,6 +26,7 @@
 #include <iostream>
 #include <fstream>
 #include "avltree.h"
+#include "vector.h"
 #include "../config.h"
 
 #define PROGNAME "kelbt"
@@ -58,6 +59,9 @@ extern bool inhibitLineDirectives;
 extern int gblErrorCount;
 extern char startDefName[];
 
+typedef Vector<const char *> ArgsVector;
+extern ArgsVector includePaths;
+
 /* Error reporting. */
 struct InputLoc;
 std::ostream &error();
