@@ -1,6 +1,7 @@
+
 #line 1 "klscan.rl"
 /*
- *  Copyright 2006 Adrian Thurston <thurston@complang.org>
+ *  Copyright 2006-2009 Adrian Thurston <thurston@complang.org>
  */
 
 /*  This file is part of Kelbt.
@@ -35,13 +36,14 @@ using std::cerr;
 using std::endl;
 
 
-#line 39 "klscan.cpp"
+#line 40 "klscan.cpp"
 static const int section_parser_start = 7;
 static const int section_parser_first_final = 7;
 static const int section_parser_error = 0;
 
 static const int section_parser_en_main = 7;
 
+
 #line 39 "klscan.rl"
 
 
@@ -54,10 +56,11 @@ struct Scanner
 		parser(0), curline(1)
 	{
 		
-#line 58 "klscan.cpp"
+#line 60 "klscan.cpp"
 	{
 	cs = section_parser_start;
 	}
+
 #line 50 "klscan.rl"
 	}
 	
@@ -68,6 +71,9 @@ struct Scanner
 	void try_token( int tokid );
 	void try_token( int tokid, char *start, char *end );
 
+	ifstream *tryOpenInclude( char **pathChecks, long &found );
+	char **makeIncludePathChecks( const char *thisFileName, const char *fileName );
+
 	/* Open an error message with the file name and line. */
 	ostream &error()
 	{
@@ -144,6 +150,76 @@ void Scanner::try_token( int tokid )
 	try_token( tokid, 0, 0 );
 }
 
+bool isAbsolutePath( const char *path )
+{
+	return path[0] == '/';
+}
+
+ifstream *Scanner::tryOpenInclude( char **pathChecks, long &found )
+{
+	char **check = pathChecks;
+	ifstream *inFile = new ifstream;
+	
+	while ( *check != 0 ) {
+		inFile->open( *check );
+		if ( inFile->is_open() ) {
+			found = check - pathChecks;
+			return inFile;
+		}
+		check += 1;
+	}
+
+	found = -1;
+	delete inFile;
+	return 0;
+}
+
+char **Scanner::makeIncludePathChecks( const char *thisFileName, const char *fileName )
+{
+	char **checks = 0;
+	long nextCheck = 0;
+	char *data = strdup(fileName);
+	long length = strlen(fileName);
+
+	/* Absolute path? */
+	if ( isAbsolutePath( data ) ) {
+		checks = new char*[2];
+		checks[nextCheck++] = data;
+	}
+	else {
+		/* Search from the the location of the current file. */
+		checks = new char *[2 + includePaths.length()];
+		const char *lastSlash = strrchr( thisFileName, '/' );
+		if ( lastSlash == 0 )
+			checks[nextCheck++] = data;
+		else {
+			long givenPathLen = (lastSlash - thisFileName) + 1;
+			long checklen = givenPathLen + length;
+			char *check = new char[checklen+1];
+			memcpy( check, thisFileName, givenPathLen );
+			memcpy( check+givenPathLen, data, length );
+			check[checklen] = 0;
+			checks[nextCheck++] = check;
+		}
+
+		/* Search from the include paths given on the command line. */
+		for ( ArgsVector::Iter incp = includePaths; incp.lte(); incp++ ) {
+			long pathLen = strlen( *incp );
+			long checkLen = pathLen + 1 + length;
+			char *check = new char[checkLen+1];
+			memcpy( check, *incp, pathLen );
+			check[pathLen] = '/';
+			memcpy( check+pathLen+1, data, length );
+			check[checkLen] = 0;
+			checks[nextCheck++] = check;
+		}
+	}
+
+	checks[nextCheck] = 0;
+	return checks;
+}
+
+
 void Scanner::try_token( int tokid, char *start, char *end )
 {
 	char *tokdata = 0;
@@ -157,14 +233,14 @@ void Scanner::try_token( int tokid, char *start, char *end )
 	}
 
 
-#line 161 "klscan.cpp"
+#line 237 "klscan.cpp"
 	{
 	if ( p == pe )
 		goto _test_eof;
 	switch ( cs )
 	{
 tr2:
-#line 155 "klscan.rl"
+#line 228 "klscan.rl"
 	{
 		//cout << "PARSER STATEMENT" << endl;
 
@@ -187,27 +263,27 @@ tr2:
 	}
 	goto st7;
 tr4:
-#line 177 "klscan.rl"
+#line 250 "klscan.rl"
 	{
-		/* FIXME: Need to check for parser. */
-		char *includeFileName = captured;
-
-		//cout << "INCLUDE STATEMENT " << includeFileName << endl;
+		/* Make the list of files to try. */
+		char **checks = makeIncludePathChecks( this->fileName, captured );
 
 		/* Open the input file for reading. */
-		ifstream *inFile = new ifstream( includeFileName );
-		istream *inStream = inFile;
-		if ( ! inFile->is_open() )
-			error() << "include: could not open " << includeFileName << " for reading" << endl;
+		long found = 0;
+		ifstream *inFile = tryOpenInclude( checks, found );
 
-		Scanner subScanner( includeFileName, *inStream, parserDict, include_depth+1 );
-		subScanner.do_scan();
-
-		delete inFile;
+		/* Open the input file for reading. */
+		if ( inFile == 0 )
+			error() << "include: could not open " << captured << " for reading" << endl;
+		else {
+			Scanner subScanner( checks[found], *inFile, parserDict, include_depth+1 );
+			subScanner.do_scan();
+			delete inFile;
+		}
 	}
 	goto st7;
 tr6:
-#line 196 "klscan.rl"
+#line 269 "klscan.rl"
 	{
 		if ( include_depth == 0 ) {
 			char *writeArg = captured;
@@ -229,7 +305,7 @@ tr6:
 	}
 	goto st7;
 tr7:
-#line 217 "klscan.rl"
+#line 290 "klscan.rl"
 	{
 		if ( parser == 0 )
 			error() << "no section name and no previously named section" << endl;
@@ -245,7 +321,7 @@ st7:
 	if ( ++p == pe )
 		goto _test_eof7;
 case 7:
-#line 249 "klscan.cpp"
+#line 325 "klscan.cpp"
 	switch( (*p) ) {
 		case 128: goto st1;
 		case 129: goto st3;
@@ -263,14 +339,14 @@ st0:
 cs = 0;
 	goto _out;
 tr0:
-#line 228 "klscan.rl"
+#line 301 "klscan.rl"
 	{ captured = tokdata; }
 	goto st2;
 st2:
 	if ( ++p == pe )
 		goto _test_eof2;
 case 2:
-#line 274 "klscan.cpp"
+#line 350 "klscan.cpp"
 	if ( (*p) == 59 )
 		goto tr2;
 	goto st0;
@@ -282,14 +358,14 @@ case 3:
 		goto tr3;
 	goto st0;
 tr3:
-#line 229 "klscan.rl"
+#line 302 "klscan.rl"
 	{ captured = tokdata; }
 	goto st4;
 st4:
 	if ( ++p == pe )
 		goto _test_eof4;
 case 4:
-#line 293 "klscan.cpp"
+#line 369 "klscan.cpp"
 	if ( (*p) == 59 )
 		goto tr4;
 	goto st0;
@@ -301,14 +377,14 @@ case 5:
 		goto tr5;
 	goto st0;
 tr5:
-#line 228 "klscan.rl"
+#line 301 "klscan.rl"
 	{ captured = tokdata; }
 	goto st6;
 st6:
 	if ( ++p == pe )
 		goto _test_eof6;
 case 6:
-#line 312 "klscan.cpp"
+#line 388 "klscan.cpp"
 	if ( (*p) == 59 )
 		goto tr6;
 	goto st0;
@@ -324,15 +400,17 @@ case 6:
 	_test_eof: {}
 	_out: {}
 	}
-#line 245 "klscan.rl"
+
+#line 318 "klscan.rl"
 
 }
 
-#line 387 "klscan.rl"
+
+#line 460 "klscan.rl"
 
 
 
-#line 336 "klscan.cpp"
+#line 414 "klscan.cpp"
 static const int klscan_start = 21;
 static const int klscan_first_final = 21;
 static const int klscan_error = 0;
@@ -342,7 +420,8 @@ static const int klscan_en_access_stmt = 40;
 static const int klscan_en_parser_def = 42;
 static const int klscan_en_main = 21;
 
-#line 390 "klscan.rl"
+
+#line 463 "klscan.rl"
 
 void Scanner::do_scan()
 {
@@ -357,14 +436,15 @@ void Scanner::do_scan()
 	bool execute = true;
 
 	
-#line 361 "klscan.cpp"
+#line 440 "klscan.cpp"
 	{
 	cs = klscan_start;
 	ts = 0;
 	te = 0;
 	act = 0;
 	}
-#line 404 "klscan.rl"
+
+#line 477 "klscan.rl"
 
 	while ( execute ) {
 		char *p = buf + have;
@@ -409,104 +489,104 @@ void Scanner::do_scan()
 		}
 
 		
-#line 413 "klscan.cpp"
+#line 493 "klscan.cpp"
 	{
 	if ( p == pe )
 		goto _test_eof;
 	switch ( cs )
 	{
 tr0:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{{p = ((te))-1;}{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr2:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p+1;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr9:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p+1;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr11:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p+1;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr33:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p+1;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr34:
-#line 383 "klscan.rl"
+#line 456 "klscan.rl"
 	{te = p+1;}
 	goto st21;
 tr43:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p;p--;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr44:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p;p--;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr46:
-#line 374 "klscan.rl"
+#line 447 "klscan.rl"
 	{te = p;p--;{
 			startSection( true );
 			{goto st42;}
 		}}
 	goto st21;
 tr47:
-#line 378 "klscan.rl"
+#line 451 "klscan.rl"
 	{te = p+1;{ 
 			startSection( false );
 			{goto st42;}
 		}}
 	goto st21;
 tr48:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p;p--;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 tr49:
-#line 363 "klscan.rl"
+#line 436 "klscan.rl"
 	{te = p;p--;{
 		if ( !generateGraphviz && gblErrorCount == 0 && include_depth == 0 )
 			outStream->write( ts, te-ts );
 	}}
 	goto st21;
 st21:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = 0;}
 	if ( ++p == pe )
 		goto _test_eof21;
 case 21:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = p;}
-#line 510 "klscan.cpp"
+#line 590 "klscan.cpp"
 	switch( (*p) ) {
 		case 0: goto tr34;
 		case 9: goto st22;
@@ -529,14 +609,14 @@ case 21:
 		goto st29;
 	goto tr33;
 tr36:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
 	goto st22;
 st22:
 	if ( ++p == pe )
 		goto _test_eof22;
 case 22:
-#line 540 "klscan.cpp"
+#line 620 "klscan.cpp"
 	switch( (*p) ) {
 		case 9: goto st22;
 		case 10: goto tr36;
@@ -545,14 +625,14 @@ case 22:
 	}
 	goto tr43;
 tr37:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st23;
 st23:
 	if ( ++p == pe )
 		goto _test_eof23;
 case 23:
-#line 556 "klscan.cpp"
+#line 636 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr44;
 		case 13: goto tr44;
@@ -593,14 +673,14 @@ case 25:
 		goto tr47;
 	goto tr46;
 tr39:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st26;
 st26:
 	if ( ++p == pe )
 		goto _test_eof26;
 case 26:
-#line 604 "klscan.cpp"
+#line 684 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr44;
 		case 13: goto tr44;
@@ -627,28 +707,28 @@ case 4:
 		goto tr0;
 	goto st3;
 tr40:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st27;
 st27:
 	if ( ++p == pe )
 		goto _test_eof27;
 case 27:
-#line 638 "klscan.cpp"
+#line 718 "klscan.cpp"
 	switch( (*p) ) {
 		case 42: goto st5;
 		case 47: goto st7;
 	}
 	goto tr44;
 tr7:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
 	goto st5;
 st5:
 	if ( ++p == pe )
 		goto _test_eof5;
 case 5:
-#line 652 "klscan.cpp"
+#line 732 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr7;
 		case 42: goto st6;
@@ -694,33 +774,33 @@ case 29:
 		goto st29;
 	goto tr49;
 tr12:
-#line 295 "klscan.rl"
+#line 368 "klscan.rl"
 	{{p = ((te))-1;}}
 	goto st30;
 tr14:
-#line 274 "klscan.rl"
+#line 347 "klscan.rl"
 	{te = p+1;}
 	goto st30;
 tr21:
-#line 273 "klscan.rl"
+#line 346 "klscan.rl"
 	{te = p+1;}
 	goto st30;
 tr23:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
-#line 273 "klscan.rl"
+#line 346 "klscan.rl"
 	{te = p+1;}
 	goto st30;
 tr50:
-#line 295 "klscan.rl"
+#line 368 "klscan.rl"
 	{te = p+1;}
 	goto st30;
 tr60:
-#line 277 "klscan.rl"
+#line 350 "klscan.rl"
 	{te = p+1;{ curly_count += 1; }}
 	goto st30;
 tr61:
-#line 279 "klscan.rl"
+#line 352 "klscan.rl"
 	{te = p+1;{ 
 			if ( --curly_count == 0 ) {
 				/* Send the entire block and free the inline_start pointer. */
@@ -731,15 +811,15 @@ tr61:
 		}}
 	goto st30;
 tr62:
-#line 275 "klscan.rl"
+#line 348 "klscan.rl"
 	{te = p;p--;}
 	goto st30;
 tr63:
-#line 295 "klscan.rl"
+#line 368 "klscan.rl"
 	{te = p;p--;}
 	goto st30;
 tr64:
-#line 288 "klscan.rl"
+#line 361 "klscan.rl"
 	{te = p+1;{
 			if ( inline_start < ts )
 				try_token( TK_Inline, inline_start, ts );
@@ -748,7 +828,7 @@ tr64:
 		}}
 	goto st30;
 tr66:
-#line 288 "klscan.rl"
+#line 361 "klscan.rl"
 	{te = p;p--;{
 			if ( inline_start < ts )
 				try_token( TK_Inline, inline_start, ts );
@@ -757,22 +837,22 @@ tr66:
 		}}
 	goto st30;
 tr67:
-#line 272 "klscan.rl"
+#line 345 "klscan.rl"
 	{te = p;p--;}
 	goto st30;
 tr68:
-#line 271 "klscan.rl"
+#line 344 "klscan.rl"
 	{te = p;p--;}
 	goto st30;
 st30:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = 0;}
 	if ( ++p == pe )
 		goto _test_eof30;
 case 30:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = p;}
-#line 776 "klscan.cpp"
+#line 856 "klscan.cpp"
 	switch( (*p) ) {
 		case 9: goto st31;
 		case 10: goto tr52;
@@ -797,14 +877,14 @@ case 30:
 		goto st39;
 	goto tr50;
 tr52:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
 	goto st31;
 st31:
 	if ( ++p == pe )
 		goto _test_eof31;
 case 31:
-#line 808 "klscan.cpp"
+#line 888 "klscan.cpp"
 	switch( (*p) ) {
 		case 9: goto st31;
 		case 10: goto tr52;
@@ -813,14 +893,14 @@ case 31:
 	}
 	goto tr62;
 tr53:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st32;
 st32:
 	if ( ++p == pe )
 		goto _test_eof32;
 case 32:
-#line 824 "klscan.cpp"
+#line 904 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr63;
 		case 13: goto tr63;
@@ -863,14 +943,14 @@ case 34:
 		goto st34;
 	goto tr66;
 tr55:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st35;
 st35:
 	if ( ++p == pe )
 		goto _test_eof35;
 case 35:
-#line 874 "klscan.cpp"
+#line 954 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr63;
 		case 13: goto tr63;
@@ -897,28 +977,28 @@ case 11:
 		goto tr12;
 	goto st10;
 tr56:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st36;
 st36:
 	if ( ++p == pe )
 		goto _test_eof36;
 case 36:
-#line 908 "klscan.cpp"
+#line 988 "klscan.cpp"
 	switch( (*p) ) {
 		case 42: goto st12;
 		case 47: goto st14;
 	}
 	goto tr63;
 tr19:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
 	goto st12;
 st12:
 	if ( ++p == pe )
 		goto _test_eof12;
 case 12:
-#line 922 "klscan.cpp"
+#line 1002 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr19;
 		case 42: goto st13;
@@ -973,25 +1053,25 @@ case 39:
 		goto st39;
 	goto tr68;
 tr71:
-#line 300 "klscan.rl"
+#line 373 "klscan.rl"
 	{te = p+1;{ 
 			try_token( ';' ); 
 			{goto st42;}
 		}}
 	goto st40;
 tr72:
-#line 299 "klscan.rl"
+#line 372 "klscan.rl"
 	{te = p;p--;{ try_token( TK_AccessData, ts, te ); }}
 	goto st40;
 st40:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = 0;}
 	if ( ++p == pe )
 		goto _test_eof40;
 case 40:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = p;}
-#line 995 "klscan.cpp"
+#line 1075 "klscan.cpp"
 	switch( (*p) ) {
 		case 0: goto st0;
 		case 59: goto tr71;
@@ -1010,38 +1090,38 @@ st0:
 cs = 0;
 	goto _out;
 tr24:
-#line 360 "klscan.rl"
+#line 433 "klscan.rl"
 	{{p = ((te))-1;}{ try_token( *ts, 0, 0 ); }}
 	goto st42;
 tr26:
-#line 334 "klscan.rl"
+#line 407 "klscan.rl"
 	{te = p+1;{ try_token( TK_String, ts+1, te-1 ); }}
 	goto st42;
 tr29:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
-#line 337 "klscan.rl"
+#line 410 "klscan.rl"
 	{te = p+1;}
 	goto st42;
 tr30:
-#line 333 "klscan.rl"
+#line 406 "klscan.rl"
 	{te = p+1;{ try_token( TK_Literal, ts+1, te-1 ); }}
 	goto st42;
 tr32:
-#line 339 "klscan.rl"
+#line 412 "klscan.rl"
 	{te = p+1;{ 
 			endSection();
 			{goto st21;}
 		}}
 	goto st42;
 tr73:
-#line 360 "klscan.rl"
+#line 433 "klscan.rl"
 	{te = p+1;{ try_token( *ts, 0, 0 ); }}
 	goto st42;
 tr75:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
-#line 346 "klscan.rl"
+#line 419 "klscan.rl"
 	{te = p+1;{
 			if ( singleLineSpec ) {
 				endSection();
@@ -1050,7 +1130,7 @@ tr75:
 		}}
 	goto st42;
 tr91:
-#line 353 "klscan.rl"
+#line 426 "klscan.rl"
 	{te = p+1;{ 
 			inline_start_line = curline;
 			inline_start = ts;
@@ -1059,19 +1139,19 @@ tr91:
 		}}
 	goto st42;
 tr93:
-#line 344 "klscan.rl"
+#line 417 "klscan.rl"
 	{te = p;p--;}
 	goto st42;
 tr94:
-#line 360 "klscan.rl"
+#line 433 "klscan.rl"
 	{te = p;p--;{ try_token( *ts, 0, 0 ); }}
 	goto st42;
 tr96:
-#line 335 "klscan.rl"
+#line 408 "klscan.rl"
 	{te = p;p--;{ try_token( TK_Number, ts, te ); }}
 	goto st42;
 tr97:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{	switch( act ) {
 	case 12:
 	{{p = ((te))-1;} try_token( KW_Commit ); }
@@ -1125,25 +1205,25 @@ tr97:
 	}
 	goto st42;
 tr98:
-#line 332 "klscan.rl"
+#line 405 "klscan.rl"
 	{te = p;p--;{ try_token( TK_Word, ts, te ); }}
 	goto st42;
 tr104:
-#line 324 "klscan.rl"
+#line 397 "klscan.rl"
 	{te = p;p--;{ 
 			try_token( KW_Access );
 			{goto st40;}
 		}}
 	goto st42;
 st42:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = 0;}
 	if ( ++p == pe )
 		goto _test_eof42;
 case 42:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{ts = p;}
-#line 1147 "klscan.cpp"
+#line 1227 "klscan.cpp"
 	switch( (*p) ) {
 		case 9: goto st43;
 		case 10: goto tr75;
@@ -1186,14 +1266,14 @@ case 43:
 	}
 	goto tr93;
 tr76:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st44;
 st44:
 	if ( ++p == pe )
 		goto _test_eof44;
 case 44:
-#line 1197 "klscan.cpp"
+#line 1277 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr94;
 		case 13: goto tr94;
@@ -1220,14 +1300,14 @@ case 16:
 		goto tr24;
 	goto st15;
 tr77:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st45;
 st45:
 	if ( ++p == pe )
 		goto _test_eof45;
 case 45:
-#line 1231 "klscan.cpp"
+#line 1311 "klscan.cpp"
 	if ( (*p) == 10 )
 		goto tr29;
 	goto st17;
@@ -1239,14 +1319,14 @@ case 17:
 		goto tr29;
 	goto st17;
 tr78:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st46;
 st46:
 	if ( ++p == pe )
 		goto _test_eof46;
 case 46:
-#line 1250 "klscan.cpp"
+#line 1330 "klscan.cpp"
 	switch( (*p) ) {
 		case 10: goto tr94;
 		case 13: goto tr94;
@@ -1274,106 +1354,106 @@ case 47:
 		goto st47;
 	goto tr96;
 tr80:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 332 "klscan.rl"
+#line 405 "klscan.rl"
 	{act = 28;}
 	goto st48;
 tr111:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 321 "klscan.rl"
+#line 394 "klscan.rl"
 	{act = 25;}
 	goto st48;
 tr115:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 308 "klscan.rl"
+#line 381 "klscan.rl"
 	{act = 12;}
 	goto st48;
 tr119:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 311 "klscan.rl"
+#line 384 "klscan.rl"
 	{act = 15;}
 	goto st48;
 tr125:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 318 "klscan.rl"
+#line 391 "klscan.rl"
 	{act = 22;}
 	goto st48;
 tr131:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 314 "klscan.rl"
+#line 387 "klscan.rl"
 	{act = 18;}
 	goto st48;
 tr137:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 317 "klscan.rl"
+#line 390 "klscan.rl"
 	{act = 21;}
 	goto st48;
 tr138:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 319 "klscan.rl"
+#line 392 "klscan.rl"
 	{act = 23;}
 	goto st48;
 tr145:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 322 "klscan.rl"
+#line 395 "klscan.rl"
 	{act = 26;}
 	goto st48;
 tr151:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 313 "klscan.rl"
+#line 386 "klscan.rl"
 	{act = 17;}
 	goto st48;
 tr153:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 309 "klscan.rl"
+#line 382 "klscan.rl"
 	{act = 13;}
 	goto st48;
 tr159:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 312 "klscan.rl"
+#line 385 "klscan.rl"
 	{act = 16;}
 	goto st48;
 tr161:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 316 "klscan.rl"
+#line 389 "klscan.rl"
 	{act = 20;}
 	goto st48;
 tr165:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 310 "klscan.rl"
+#line 383 "klscan.rl"
 	{act = 14;}
 	goto st48;
 tr167:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 315 "klscan.rl"
+#line 388 "klscan.rl"
 	{act = 19;}
 	goto st48;
 tr171:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
-#line 320 "klscan.rl"
+#line 393 "klscan.rl"
 	{act = 24;}
 	goto st48;
 st48:
 	if ( ++p == pe )
 		goto _test_eof48;
 case 48:
-#line 1377 "klscan.cpp"
+#line 1457 "klscan.cpp"
 	if ( (*p) == 95 )
 		goto tr80;
 	if ( (*p) < 65 ) {
@@ -1491,14 +1571,14 @@ case 54:
 		goto tr80;
 	goto tr104;
 tr106:
-#line 255 "klscan.rl"
+#line 328 "klscan.rl"
 	{ curline++; }
 	goto st55;
 st55:
 	if ( ++p == pe )
 		goto _test_eof55;
 case 55:
-#line 1502 "klscan.cpp"
+#line 1582 "klscan.cpp"
 	switch( (*p) ) {
 		case 9: goto st55;
 		case 10: goto tr106;
@@ -2516,14 +2596,14 @@ case 114:
 		goto tr80;
 	goto tr98;
 tr92:
-#line 1 "klscan.rl"
+#line 1 "NONE"
 	{te = p+1;}
 	goto st115;
 st115:
 	if ( ++p == pe )
 		goto _test_eof115;
 case 115:
-#line 2527 "klscan.cpp"
+#line 2607 "klscan.cpp"
 	if ( (*p) == 37 )
 		goto st20;
 	goto tr94;
@@ -2771,7 +2851,8 @@ case 20:
 
 	_out: {}
 	}
-#line 448 "klscan.rl"
+
+#line 521 "klscan.rl"
 
 		/* Check if we failed. */
 		if ( cs == klscan_error ) {
